# from typing import Optional, TypeVar, TypedDict, Union
# from enum import Enum

int64 = int
int32 = int
int16 = int
int8 = int

nat = int
nat64 = int
nat32 = int
nat16 = int
nat8 = int

float64 = float
float32 = float

text = str

# blob = bytes

# null = None

# my_list = list[int]

# T = TypeVar('T')
# opt = Optional[T]

# list[T]

# Variant = Enum
# Record = TypedDict

# my_tuple = tuple[str, str]
# Record = TypedDict

# def record(func: object) -> TypedDict:
#     return func

# # @record
# class MyRecord(Record):
#     prop1: str

# temp: MyRecord = {
#     'prop1': 'hello'
# }
# class MyRecord(TypedDict):
#     prop1: str
#     prop2: int

# class MyVariant1:
#     prop1: str

# class MyVariant2:
#     prop2: int

# variant = MyVariant1 | MyVariant2

# your_variant: variant = MyVariant1()

# if your_variant.prop2

# def test(the_variant: variant):
#     return the_variant.prop2

# class MyVariant:
#     prop1: str
#     prop2: int

# my_variant: MyVariant = MyVariant()

# class MyRecord(Record):
#     prop1: str
#     prop2: int

# class MyVariant(Variant):
#     Prop1 = opt[str]
#     Prop2 = opt[int]

# TODO this should work for records
# MyRecord = Record('MyRecord', {
#     'prop1': str,
#     'prop2': int
# })

# TODO we just need to figure out how to get the typechecking to work well on the variant
# TODO if I can make my own class called variant that takes in these types as params and sets them to opt, we might be good to go
# TODO also look into an enum
# MyVariant = Variant('MyVariant', {
#     'prop1': opt[str],
#     'prop2': opt[int]
# })

# my_record = dict[
#     'hello': int32
# ]

# object

# TODO we probably want to use decorators for variants and records

def query(func: object):
    return func

def update(func: object):
    return func

# def record(func: object):
#     return func

# def variant(func: object):
#     return func

# def test_variant(my_variant: MyVariant) -> MyVariant:

#     # TODO working on the Enum, we need it to carry a dynamic value
#     another_variant: MyVariant = MyVariant(Prop1 = 'hello')

#     return my_variant.Prop2

# def monkey(record: MyRecord) -> str:
#     temp: MyRecord = {
#         'prop1': 'yes',
#         'prop2': 32
#     }

#     # temp: MyRecord = new MyRecord()

#     return temp['prop1']



# type: ignore

# TODO just move all of the bash stuff into here probably

import kybra
import modulegraph.modulegraph
import os
import shutil
import subprocess
import sys

canister_name = sys.argv[1]
py_entry_file_path = sys.argv[2]
did_path = sys.argv[3]
compiler_path = os.path.dirname(kybra.__file__) + '/compiler'
canister_path=f'.dfx/kybra/{canister_name}'
build_sh_path = compiler_path + '/build.sh'

shutil.copytree(compiler_path, canister_path, dirs_exist_ok=True)

path = list(filter(lambda x: x.startswith(os.getcwd()), sys.path)) + [os.path.dirname(py_entry_file_path)]

graph = modulegraph.modulegraph.ModuleGraph(path)
entry_point = graph.run_script(py_entry_file_path)

python_source_path = f'{canister_path}/python_source'

if os.path.exists(python_source_path):
    shutil.rmtree(python_source_path)

os.makedirs(python_source_path)

for node in graph.flatten(start=entry_point):
    if type(node) == modulegraph.modulegraph.Script:
        shutil.copy(node.filename, f'{python_source_path}/{os.path.basename(node.filename)}')

    if type(node) == modulegraph.modulegraph.SourceModule:
        shutil.copy(node.filename, f'{python_source_path}/{os.path.basename(node.filename)}')

    if type(node) == modulegraph.modulegraph.Package:
        shutil.copytree(node.packagepath[0], f'{python_source_path}/{node.identifier}', dirs_exist_ok=True)

subprocess.call([build_sh_path, canister_name, py_entry_file_path, did_path, compiler_path])
